/* BDF - Copyright (c) 2015-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.multi;

import fr.exemole.bdfserver.api.storage.ResourceStorage;
import fr.exemole.bdfserver.multi.commands.IniCommand;
import fr.exemole.bdfserver.multi.htmlproducers.AdminJsHtmlProducer;
import fr.exemole.bdfserver.multi.htmlproducers.IndexMultiHtmlProducer;
import fr.exemole.bdfserver.multi.htmlproducers.IniFormHtmlProducer;
import fr.exemole.bdfserver.multi.htmlproducers.IniResultHtmlProducer;
import fr.exemole.bdfserver.multi.htmlproducers.LoginMultiHtmlProducer;
import fr.exemole.bdfserver.multi.htmlproducers.MultiHtmlProducer;
import fr.exemole.bdfserver.multi.jsonproducers.MultiJsonProducer;
import fr.exemole.bdfserver.multi.jsonproducers.MultiJsonProducerFactory;
import fr.exemole.bdfserver.multi.jsonproducers.PingJsonProducer;
import fr.exemole.bdfserver.multi.streamproducers.SynthesisOdsProducer;
import fr.exemole.bdfserver.tools.docstream.DocStreamFactory;
import java.text.ParseException;
import java.util.List;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.io.DocStream;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.mimetype.MimeTypeUtils;
import net.mapeadores.util.request.RequestConstants;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.request.RequestUtils;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.ServletUtils;
import net.mapeadores.util.servlets.handlers.CommandMessageResponseHandler;
import net.mapeadores.util.servlets.handlers.JsonResponseHandler;
import net.mapeadores.util.text.RelativePath;


/**
 *
 * @author Vincent Calame
 */
public final class MultiResponseHandlerFactory {

    private MultiResponseHandlerFactory() {

    }

    public static ResponseHandler getResourceResponseHandler(Multi multi, String path) {
        List<ResourceStorage> webappsResourceStorageList = multi.getWebappsResourceStorageList();
        RelativePath relativePath;
        try {
            relativePath = RelativePath.parse(path);
        } catch (ParseException pe) {
            return null;
        }
        DocStream docStream = DocStreamFactory.getResourceDocStream(webappsResourceStorageList, relativePath, MimeTypeUtils.DEFAULT_RESOLVER);
        if (docStream == null) {
            return null;
        }
        return ServletUtils.wrap(docStream);
    }

    public static ResponseHandler getIniResponseHandler(Multi multi, RequestMap requestMap) {
        String cmd = requestMap.getParameter(RequestConstants.COMMAND_PARAMETER);
        IniCommand.Result result = null;
        if ((cmd != null) && (cmd.equals(IniCommand.COMMAND_NAME))) {
            IniCommand iniCommand = new IniCommand(requestMap, multi);
            result = iniCommand.doCommand();
        }
        MultiHtmlProducer htmlProducer;
        if ((result != null) && (!result.hasError())) {
            htmlProducer = new IniResultHtmlProducer(multi, result);
        } else {
            htmlProducer = new IniFormHtmlProducer(multi, requestMap);
        }
        if (result != null) {
            htmlProducer.setCommandMessage(result.getCommandMessage());
        }
        return ServletUtils.wrap(htmlProducer);
    }

    public static ResponseHandler getAdminJsRespondeHandler(Multi multi) {
        AdminJsHtmlProducer htmlProducer = new AdminJsHtmlProducer(multi);
        return ServletUtils.wrap(htmlProducer);
    }

    public static ResponseHandler getAdminNoscriptRespondeHandler(Multi multi, short check, RequestMap requestMap) {
        MultiHtmlProducer htmlProducer;
        CommandMessage commandMessage = null;
        switch (check) {
            case MultiConf.AUTHREQUIRED_STATE:
                htmlProducer = new LoginMultiHtmlProducer(multi);
                commandMessage = LogUtils.error("_ warning.multi.authentification_required");
                break;
            case MultiConf.AUTHFAILED_STATE:
                htmlProducer = new LoginMultiHtmlProducer(multi);
                commandMessage = LogUtils.error("_ warning.multi.authentification_failed");
                break;
            case MultiConf.CONNECTED_STATE:
                htmlProducer = new IndexMultiHtmlProducer(multi);
                commandMessage = MultiAdminCommands.run(multi, requestMap);
                break;
            default:
                throw new SwitchException("Unknown check: " + check);
        }
        if (commandMessage != null) {
            htmlProducer.setCommandMessage(commandMessage);
        }
        return ServletUtils.wrap(htmlProducer);
    }

    public static ResponseHandler getJsonRespondeHandler(Multi multi, short check, RequestMap requestMap, String output) {
        short jsonType = RequestUtils.getJsonType(requestMap);
        if (output == null) {
            output = MultiConstants.PING_JSON;
        }
        String callback = requestMap.getParameter(RequestConstants.CALLBACK_PARAMETER);
        MultiJsonProducer jsonProducer = null;
        switch (check) {
            case MultiConf.AUTHREQUIRED_STATE:
                jsonProducer = new PingJsonProducer(multi, false);
                break;
            case MultiConf.AUTHFAILED_STATE:
                jsonProducer = new PingJsonProducer(multi, false);
                jsonProducer.setCommandMessage(LogUtils.error("_ warning.multi.authentification_failed"));
                break;
            case MultiConf.CONNECTED_STATE:
                CommandMessage commandMessage = MultiAdminCommands.run(multi, requestMap);
                jsonProducer = MultiJsonProducerFactory.getJsonProducer(multi, output, commandMessage, requestMap);
                break;
            default:
                throw new SwitchException("Unknown check: " + check);
        }
        return new JsonResponseHandler(jsonProducer, jsonType, callback);
    }

    public static ResponseHandler getStreamRespondeHandler(Multi multi, String output) {
        switch (output) {
            case MultiConstants.SYNTHESIS_STREAM:
                if (multi.isAuthentificationSharingAllowed()) {
                    return ServletUtils.wrap(new SynthesisOdsProducer(multi, MultiUtils.getSynthesis(multi)));
                } else {
                    return CommandMessageResponseHandler.getHtmlInstance(multi.getMultiConf().getMessageLocalisation(), LogUtils.error("_ error.unsupported.sharingnotallowed"));
                }
            default:
                return CommandMessageResponseHandler.getHtmlInstance(multi.getMultiConf().getMessageLocalisation(), LogUtils.error("_ error.unknown.parametervalue", RequestConstants.STREAM_PARAMETER, output));
        }
    }

}
