/* BdfServer - Copyright (c) 2009-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.ficheform;

import fr.exemole.bdfserver.api.ficheform.FicheFormParameters;
import fr.exemole.bdfserver.api.ficheform.FormElement;
import fr.exemole.bdfserver.api.ficheform.ThesaurusIncludeElement;
import fr.exemole.bdfserver.api.namespaces.RoleSpace;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.api.ui.FieldUi;
import fr.exemole.bdfserver.api.ui.IncludeUi;
import fr.exemole.bdfserver.api.ui.SubsetIncludeUi;
import fr.exemole.bdfserver.api.ui.options.InputOption;
import fr.exemole.bdfserver.api.ui.options.ValueOption;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.ficheform.builders.FicheStyleThesaurusIncludeElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.HiddenFieldElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.HiddenIncludeElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.ItemFieldElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.ListThesaurusIncludeElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.NotEditableThesaurusIncludeElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.TextFieldElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.TextThesaurusIncludeElementBuilder;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import net.fichotheque.Fichotheque;
import net.fichotheque.Subset;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.fiche.FicheBlocks;
import net.fichotheque.corpus.fiche.FicheItem;
import net.fichotheque.corpus.fiche.FicheItems;
import net.fichotheque.corpus.metadata.CorpusField;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.croisement.Croisements;
import net.fichotheque.croisement.Liaison;
import net.fichotheque.include.ExtendedIncludeKey;
import net.fichotheque.include.IncludeKey;
import net.fichotheque.pointeurs.FichePointeur;
import net.fichotheque.pointeurs.SubsetItemPointeur;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.syntax.FicheblockSyntax;
import net.fichotheque.syntax.FormSyntax;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.utils.CorpusMetadataUtils;
import net.fichotheque.utils.CroisementUtils;
import net.fichotheque.utils.ThesaurusUtils;
import net.mapeadores.util.annotation.Nullable;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.text.SeparatorOptions;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public final class FicheFormUtils {

    private FicheFormUtils() {
    }

    public static FormElement.Field toFormElement(FichePointeur fichePointeur, FicheFormParameters ficheFormParameters, CorpusField corpusField, FieldUi fieldUi) {
        return toFormElement(fichePointeur, ficheFormParameters, corpusField, fieldUi, null);
    }

    public static FormElement.Field toFormElement(FichePointeur fichePointeur, FicheFormParameters ficheFormParameters, CorpusField corpusField, FieldUi fieldUi, @Nullable String defaultValue) {
        if (corpusField.getFieldKey().ownsToCorpsdefiche()) {
            return TextFieldElementBuilder.build(fichePointeur, ficheFormParameters, corpusField, fieldUi, defaultValue)
                    .toTextFieldElement();
        }
        InputOption inputOption = fieldUi.getInputOption();
        switch (inputOption.getInputType()) {
            case InputOption.INPUT_HIDDEN:
                return HiddenFieldElementBuilder.build(fichePointeur, ficheFormParameters, corpusField, fieldUi, defaultValue)
                        .toHiddenFieldElement();
            default:
                return ItemFieldElementBuilder.build(fichePointeur, ficheFormParameters, corpusField, fieldUi, defaultValue)
                        .toItemFieldElement();
        }
    }

    public static FormElement.Include toThesaurusIncludeElement(FichePointeur fichePointeur, FicheFormParameters ficheFormParameters, SubsetIncludeUi includeUi) {
        InputOption inputOption = includeUi.getInputOption();
        switch (inputOption.getInputType()) {
            case InputOption.INPUT_CHECK: {
                ListThesaurusIncludeElementBuilder builder = ListThesaurusIncludeElementBuilder.check(fichePointeur, ficheFormParameters, includeUi);
                if (builder != null) {
                    return builder.toListThesaurusIncludeElement();
                } else {
                    return null;
                }
            }
            case InputOption.INPUT_RADIO:
            case InputOption.INPUT_SELECT: {
                ListThesaurusIncludeElementBuilder builder = ListThesaurusIncludeElementBuilder.check(fichePointeur, ficheFormParameters, includeUi);
                if (builder != null) {
                    ThesaurusIncludeElement.List ltie = builder.toListThesaurusIncludeElement();
                    if (ltie.getSelectedMotcleSet().size() > 1) {
                        TextThesaurusIncludeElementBuilder ttieb2 = TextThesaurusIncludeElementBuilder.check(fichePointeur, ficheFormParameters, includeUi);
                        if (ttieb2 != null) {
                            return ttieb2.toTextThesaurusIncludeElement();
                        } else {
                            return null;
                        }
                    } else {
                        return ltie;
                    }
                } else {
                    return null;
                }
            }
            case InputOption.INPUT_TEXT:
            case InputOption.INPUT_SUGGESTION: {
                TextThesaurusIncludeElementBuilder builder = TextThesaurusIncludeElementBuilder.check(fichePointeur, ficheFormParameters, includeUi);
                if (builder != null) {
                    return builder.toTextThesaurusIncludeElement();
                } else {
                    return null;
                }
            }
            case InputOption.INPUT_HIDDEN: {
                HiddenIncludeElementBuilder builder = HiddenIncludeElementBuilder.check(fichePointeur, ficheFormParameters, includeUi);
                if (builder != null) {
                    return builder.toHiddenIncludeElement();
                } else {
                    return null;
                }
            }
            case InputOption.INPUT_NOTEDITABLE: {
                NotEditableThesaurusIncludeElementBuilder builder = NotEditableThesaurusIncludeElementBuilder.check(fichePointeur, ficheFormParameters, includeUi);
                if (builder != null) {
                    return builder.toNotEditableIndexationElement();
                } else {
                    return null;
                }
            }
            case InputOption.INPUT_FICHESTYLE: {
                FicheStyleThesaurusIncludeElementBuilder builder = FicheStyleThesaurusIncludeElementBuilder.check(fichePointeur, ficheFormParameters, includeUi);
                if (builder != null) {
                    return builder.toFicheStyleThesaurusIncludeElement();
                } else {
                    return null;
                }
            }
            default:
                throw new SwitchException("wrong inputOption.getInputType() = " + inputOption.getInputType());
        }
    }

    public static Collection<Liaison> initMotcleList(SubsetItemPointeur subsetItemPointeur, Thesaurus thesaurus, IncludeKey includeKey) {
        Croisements motcleCroisements = subsetItemPointeur.getCroisements(thesaurus);
        return CroisementUtils.filter(motcleCroisements, includeKey.getMode(), includeKey.getPoidsFilter());
    }

    public static SubsetItemPointeur checkMasterPointeur(FichePointeur fichePointeur, FicheFormParameters ficheFormParameters, ExtendedIncludeKey extentedIncludeKey) {
        if (extentedIncludeKey.isMaster()) {
            Subset masterSubset = fichePointeur.getCorpus().getMasterSubset();
            if (masterSubset != null) {
                SubsetItemPointeur masterPointeur = fichePointeur.getRattachementPointeur(masterSubset.getSubsetKey());
                if (masterPointeur.isEmpty()) {
                    masterPointeur.setCurrentSubsetItem(ficheFormParameters.getMasterSubsetItem());
                }
                return masterPointeur;
            } else {
                return fichePointeur;
            }

        } else {
            return fichePointeur;
        }
    }

    public static String getStringValue(FichePointeur fichePointeur, CorpusField corpusField, FicheFormParameters ficheFormParameters) {
        FormSyntax.Parameters parameters = getParameters(ficheFormParameters, corpusField);
        Fichotheque fichotheque = fichePointeur.getFichotheque();
        FieldKey fieldKey = corpusField.getFieldKey();
        Object obj = fichePointeur.getValue(fieldKey);
        if (obj == null) {
            return "";
        }
        if (obj instanceof Integer) {
            return obj.toString();
        }
        if (obj instanceof String) {
            return FormSyntax.escapeString((String) obj);
        }
        if (obj instanceof FicheItem) {
            return FormSyntax.toString((FicheItem) obj, fichotheque, parameters);
        }
        if (obj instanceof FicheItems) {
            boolean isBlock = CorpusMetadataUtils.isBlockDisplayInformationField(corpusField);
            SeparatorOptions separatorOptions = (isBlock) ? ficheFormParameters.getBlockSeparatorOptions() : ficheFormParameters.getInlineSeparatorOptions();
            return FormSyntax.toString((FicheItems) obj, fichotheque, separatorOptions, parameters);
        }
        if (obj instanceof FicheBlocks) {
            return FicheblockSyntax.toString((FicheBlocks) obj, ficheFormParameters.getFicheBlockFormSyntaxParameters());
        }
        throw new SwitchException("Unexpected instance of " + obj.getClass().getName());
    }

    public static FormSyntax.Parameters getParameters(FicheFormParameters ficheFormParameters, CorpusField corpusField) {
        FormSyntax.Parameters currentParameters = ficheFormParameters.getFicheItemFormSyntaxParameters();
        SubsetKey defaultSphereKey = corpusField.getFieldOptions().getDefaultSphereKey();
        if (defaultSphereKey == null) {
            return currentParameters;
        }
        return FormSyntax.parameters(currentParameters).defautSphereKey(defaultSphereKey);
    }

    public static String getStringValue(Thesaurus thesaurus, Collection<Liaison> motcleLiaisonList, boolean ignorePoids, FicheFormParameters ficheFormParameters) {
        if (motcleLiaisonList.isEmpty()) {
            return "";
        }
        boolean withIdalpha = ThesaurusUtils.isIdalphaType(thesaurus);
        boolean isBabelien = ThesaurusUtils.isBabelienType(thesaurus);
        Lang lang = BdfServerUtils.checkLangDisponibility(ficheFormParameters.getBdfServer(), thesaurus, ficheFormParameters.getWorkingLang());
        SeparatorOptions separatorOptions = ficheFormParameters.getInlineSeparatorOptions();
        StringBuilder buf = new StringBuilder();
        boolean next = false;
        for (Liaison liaison : motcleLiaisonList) {
            if (next) {
                appendSeparator(buf, separatorOptions);
            } else {
                next = true;
            }
            Motcle motcle = (Motcle) liaison.getSubsetItem();
            int poids = liaison.getLien().getPoids();
            if (withIdalpha) {
                buf.append(motcle.getIdalpha());
            } else if (isBabelien) {
                buf.append(ThesaurusUtils.getBabelienLabel(motcle).getLabelString());
            } else {
                Label label = LabelUtils.getLangPartCheckedLabel(motcle.getLabels(), lang);
                if (label != null) {
                    buf.append(label.getLabelString());
                } else {
                    buf.append(String.valueOf(motcle.getId()));
                }
            }
            if (!ignorePoids) {
                appendPoids(buf, poids);
            }
        }
        if (separatorOptions.isLastInclude()) {
            appendSeparator(buf, separatorOptions);
        }
        return buf.toString();
    }

    public static Set<Motcle> getDefaultMotcleSet(FicheFormParameters ficheFormParameters, IncludeUi includeUi, Thesaurus thesaurus) {
        String defaultValue = FicheFormUtils.getDefVal(ficheFormParameters, includeUi);
        if (defaultValue.length() == 0) {
            return Collections.emptySet();
        }
        String[] tokens = StringUtils.getTokens(defaultValue, ';', StringUtils.EMPTY_EXCLUDE);
        Set<Motcle> defaultSet = new HashSet<Motcle>();
        boolean withIdalpha = ThesaurusUtils.isIdalphaType(thesaurus);
        for (String token : tokens) {
            if (withIdalpha) {
                Motcle motcle = thesaurus.getMotcleByIdalpha(token);
                if (motcle != null) {
                    defaultSet.add(motcle);
                }
            } else {
                try {
                    int motcleid = Integer.parseInt(token);
                    Motcle motcle = thesaurus.getMotcleById(motcleid);
                    if (motcle != null) {
                        defaultSet.add(motcle);
                    }
                } catch (NumberFormatException nfe) {
                }
            }
        }
        return defaultSet;
    }

    public static Set<Motcle> getMotcleSet(Collection<Liaison> liaisonCollection) {
        if (liaisonCollection.isEmpty()) {
            return Collections.emptySet();
        }
        Set<Motcle> set = new HashSet<Motcle>();
        for (Liaison liaison : liaisonCollection) {
            set.add((Motcle) liaison.getSubsetItem());
        }
        return set;
    }

    private static void appendPoids(StringBuilder buf, int poids) {
        if (poids > 1) {
            switch (poids) {
                case 2:
                    buf.append('+');
                    break;
                case 3:
                    buf.append("++");
                    break;
                case 4:
                    buf.append("+++");
                    break;
                default:
                    buf.append(" <");
                    buf.append(poids);
                    buf.append('>');
                    break;
            }
        }
    }

    private static void appendSeparator(StringBuilder stringBuffer, SeparatorOptions separatorOptions) {
        stringBuffer.append(separatorOptions.getSeparator());
        if (separatorOptions.isSpaceInclude()) {
            stringBuffer.append(' ');
        }
    }

    public static int checkRows(int rows, int valueLength) {
        if (rows > 1) {
            if (valueLength > (rows * 140)) {
                rows = rows + 2;
            }
            return rows;
        }
        if (valueLength > 80) {
            return 3;
        }
        return 1;
    }

    public static boolean isMandatory(IncludeUi includeUi) {
        ValueOption valueOption = includeUi.getValueOption();
        if (valueOption == null) {
            return false;
        }
        return valueOption.isMandatory();
    }

    public static boolean isMandatory(FieldUi fieldUi) {
        ValueOption valueOption = fieldUi.getValueOption();
        if (valueOption == null) {
            return false;
        }
        return valueOption.isMandatory();
    }

    public static String getDefVal(FicheFormParameters ficheFormParameters, IncludeUi includeUi) {
        ValueOption valueOption = includeUi.getValueOption();
        if (valueOption == null) {
            return "";
        }
        return replaceGenericParameters(ficheFormParameters, valueOption.getDefaultValue());
    }

    public static String getDefVal(FichePointeur fichePointeur, FicheFormParameters ficheFormParameters, FieldUi fieldUi) {
        FieldKey fieldKey = fieldUi.getFieldKey();
        if (fieldKey == FieldKey.REDACTEURS) {
            Redacteur redacteur = ficheFormParameters.getDefaultRedacteur();
            if (redacteur == null) {
                return "";
            }
            String value = redacteur.getLogin();
            SeparatorOptions separatorOptions = ficheFormParameters.getInlineSeparatorOptions();
            if (separatorOptions.isLastInclude()) {
                value = value + separatorOptions.getSeparator();
                if (separatorOptions.isSpaceInclude()) {
                    value = value + " ";
                }
            }
            return value;
        } else {
            ValueOption valueOption = fieldUi.getValueOption();
            if (valueOption != null) {
                String defaultValue = valueOption.getDefaultValue();
                defaultValue = replaceGenericParameters(ficheFormParameters, defaultValue);
                if (defaultValue.length() > 0) {
                    if (fieldKey.isInformationFieldKey()) {
                        CorpusField corpusField = getCorpusField(fichePointeur, fieldUi);
                        if ((corpusField == null) || (!CorpusMetadataUtils.isBlockDisplayInformationField(corpusField))) {
                            SeparatorOptions separatorOptions = ficheFormParameters.getInlineSeparatorOptions();
                            if (separatorOptions.isLastInclude()) {
                                defaultValue = defaultValue + separatorOptions.getSeparator();
                                if (separatorOptions.isSpaceInclude()) {
                                    defaultValue = defaultValue + " ";
                                }
                            }
                        }
                    }
                    return defaultValue;
                }
            }
            return "";
        }
    }

    public static String replaceGenericParameters(FicheFormParameters ficheFormParameters, String s) {
        int length = s.length();
        if (length < 9) {
            return s;
        }
        int idx1 = s.indexOf("$PARAM_");
        if (idx1 == -1) {
            return s;
        }
        StringBuilder buf = new StringBuilder();
        buf.append(s.substring(0, idx1));
        while (true) {
            int idx2 = s.indexOf('$', idx1 + 1);
            if (idx2 == -1) {
                buf.append(s.substring(idx1));
                break;
            }
            String name = s.substring(idx1 + 7, idx2);
            String val = ficheFormParameters.getSupplementaryParameter(name);
            if (val != null) {
                buf.append(val);
            } else {
                buf.append("??PARAM_").append(name).append("??");
            }
            idx1 = s.indexOf("$PARAM_", idx2 + 1);
            if (idx1 == -1) {
                buf.append(s.substring(idx2 + 1));
                break;
            } else {
                buf.append(s.substring(idx2 + 1, idx1));
            }
        }
        return buf.toString();
    }

    /*public static boolean hasAccess(FicheFormParameters ficheFormParameters, IncludeUi includeUi) {
        PermissionSummary permissionSummary = ficheFormParameters.getPermissionSummary();
        ExtendedIncludeKey includeKey = includeUi.getExtendedIncludeKey();
        if (includeKey == null) {
            return true;
        }
        if (!permissionSummary.hasAccess(includeKey.getSubsetKey())) {
            return false;
        }
        if (!RoleSpace.canWrite(permissionSummary, includeUi.getAttributes())) {
            return false;
        }
        return true;
    }*/

    public static boolean hasAccess(FicheFormParameters ficheFormParameters, SubsetIncludeUi includeUi) {
        PermissionSummary permissionSummary = ficheFormParameters.getPermissionSummary();
        if (!permissionSummary.hasAccess(includeUi.getSubsetKey())) {
            return false;
        }
        if (!RoleSpace.canWrite(permissionSummary, includeUi.getAttributes())) {
            return false;
        }
        return true;
    }

    public static CorpusField getCorpusField(FichePointeur fichePointeur, FieldUi fieldUi) {
        return fichePointeur.getCorpus().getCorpusMetadata().getCorpusField(fieldUi.getFieldKey());
    }

}
