/* BdfServer_Commands - Copyright (c) 2011-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.corpus;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.BdfServerEditor;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.ui.FieldUi;
import fr.exemole.bdfserver.api.ui.SpecialIncludeUi;
import fr.exemole.bdfserver.api.ui.SubsetIncludeUi;
import fr.exemole.bdfserver.api.ui.UiComponent;
import fr.exemole.bdfserver.api.ui.UiComponents;
import fr.exemole.bdfserver.api.ui.options.InputOption;
import fr.exemole.bdfserver.api.ui.options.LimitOption;
import fr.exemole.bdfserver.api.ui.options.ValueOption;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.ui.components.FieldUiBuilder;
import fr.exemole.bdfserver.tools.ui.components.IncludeUiBuilder;
import fr.exemole.bdfserver.tools.ui.options.InputOptionBuilder;
import fr.exemole.bdfserver.tools.ui.options.LimitOptionBuilder;
import fr.exemole.bdfserver.tools.ui.options.ValueOptionBuilder;
import java.util.ArrayList;
import java.util.List;
import net.fichotheque.FichothequeConstants;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class UiComponentOptionsCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "UiComponentOptions";
    public final static String COMMANDKEY = "_ CRP-19";
    public final static String WIDTH_PARAMSUFFIX = "|width";
    public final static String ROWS_PARAMSUFFIX = "|rows";
    public final static String INPUT_PARAMSUFFIX = "|input";
    public final static String NONEW_PARAMSUFFIX = "|nonew";
    public final static String DEFAULT_PARAMSUFFIX = "|default";
    public final static String MANDATORY_PARAMSUFFIX = "|mandatory";
    private Corpus corpus;
    private UiComponents uiComponents;
    private List<UiComponent> uiComponentList;

    public UiComponentOptionsCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() {
        putResultObject(CORPUS_OBJ, corpus);
        BdfServerEditor bdfServerEditor = bdfServer.getBdfServerEditor();
        boolean done = false;
        for (UiComponent componentUi : uiComponentList) {
            boolean stepDone = bdfServerEditor.putComponentUi(uiComponents, componentUi);
            if (stepDone) {
                done = true;
            }
        }
        if (done) {
            bdfServerEditor.saveChanges();
            setDone("_ done.corpus.uicomponentoptions");
        }
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        corpus = BdfInstructionUtils.getMandatoryCorpus(fichotheque, requestMap);
        checkSubsetAdminPermission(corpus);
        uiComponents = bdfServer.getUiManager().getMainUiComponents(corpus);
        uiComponentList = new ArrayList<UiComponent>();
        for (UiComponent uiComponent : uiComponents.getUiComponentList()) {
            if (uiComponent instanceof FieldUi) {
                FieldUi fieldUi = (FieldUi) uiComponent;
                String racine = fieldUi.getFieldKey().toString();
                uiComponentList.add(FieldUiBuilder.derive(fieldUi)
                        .setInputOption(update(requestMap, racine, fieldUi.getInputOption(), false))
                        .setValueOption(update(requestMap, racine, fieldUi.getValueOption()))
                        .toFieldUi());
            } else if (uiComponent instanceof SpecialIncludeUi) {
                SpecialIncludeUi includeUi = (SpecialIncludeUi) uiComponent;
                if ((includeUi.getName().equals(FichothequeConstants.LIAGE_NAME))) {
                    uiComponentList.add(IncludeUiBuilder.derive(includeUi)
                            .setLimitOption(update(requestMap, includeUi.getName(), includeUi.getLimitOption()))
                            .toIncludeUi());
                }
            } else if (uiComponent instanceof SubsetIncludeUi) {
                SubsetIncludeUi includeUi = (SubsetIncludeUi) uiComponent;
                if (includeUi.getCategory() == SubsetKey.CATEGORY_THESAURUS) {
                    uiComponentList.add(IncludeUiBuilder.derive(includeUi)
                            .setInputOption(update(requestMap, includeUi.getName(), includeUi.getInputOption(), true))
                            .setValueOption(update(requestMap, includeUi.getName(), includeUi.getValueOption()))
                            .toIncludeUi());
                }
            }
        }
        return true;
    }

    private LimitOption update(RequestMap requestMap, String key, LimitOption limitOption) throws BdfInstructionException {
        boolean nonewAllowed = requestMap.isTrue(key + NONEW_PARAMSUFFIX);
        return LimitOptionBuilder.derive(limitOption)
                .setNoNewAllowed(nonewAllowed)
                .toLimitOption();
    }

    private InputOption update(RequestMap requestMap, String key, InputOption inputOption, boolean withInputType) throws BdfInstructionException {
        InputOptionBuilder inputOptionBuilder = InputOptionBuilder.derive(inputOption);
        String width = requestMap.getParameter(key + WIDTH_PARAMSUFFIX);
        if (width != null) {
            try {
                inputOptionBuilder.setWidthType(width);
            } catch (IllegalArgumentException iae) {
                throw new BdfInstructionException(BdfInstructionUtils.unknownParameterValue(key + WIDTH_PARAMSUFFIX, width));
            }
        }
        String rows = requestMap.getParameter(key + ROWS_PARAMSUFFIX);
        if (rows != null) {
            try {
                int r = Integer.parseInt(rows);
                if (r > 0) {
                    inputOptionBuilder.setRows(r);
                }
            } catch (NumberFormatException nfe) {
            }
        }
        if (withInputType) {
            String param = requestMap.getParameter(key + INPUT_PARAMSUFFIX);
            if (param != null) {
                try {
                    inputOptionBuilder.setInputType(param);
                } catch (IllegalArgumentException iae) {
                    throw new BdfInstructionException(BdfInstructionUtils.unknownParameterValue(key + INPUT_PARAMSUFFIX, param));
                }
            }
        }
        return inputOptionBuilder.toInputOption();
    }

    private ValueOption update(RequestMap requestMap, String key, ValueOption valueOption) throws BdfInstructionException {
        if (valueOption == null) {
            return null;
        }
        boolean mandatory = requestMap.isTrue(key + MANDATORY_PARAMSUFFIX);
        String defaultValue = requestMap.getParameter(key + DEFAULT_PARAMSUFFIX);
        if (defaultValue != null) {
            defaultValue = StringUtils.cleanString(defaultValue);
        }
        return ValueOptionBuilder.init()
                .setMandatory(mandatory)
                .setDefaultValue(defaultValue)
                .toValueOption();
    }

}
