/* BdfServer_Commands - Copyright (c) 2012-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.exportation;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.exportation.table.FicheTableParameters;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.managers.TableExportManager;
import fr.exemole.bdfserver.tools.exportation.table.FicheTableDefBuilder;
import fr.exemole.bdfserver.tools.exportation.table.MotcleTableDefBuilder;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.interaction.FicheTableParametersParser;
import java.text.ParseException;
import java.util.LinkedHashSet;
import java.util.Set;
import net.fichotheque.EditOrigin;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.exportation.table.TableDef;
import net.fichotheque.exportation.table.TableExportDescription;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.utils.EligibilityUtils;
import net.fichotheque.utils.FichothequeUtils;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class TableExportCreationCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "TableExportCreation";
    public final static String COMMANDKEY = "_ EXP-11";
    public final static String NEWTABLEEXPORT_PARAMNAME = "newtableexport";
    public final static String CORPUSLIST_PARAMNAME = "corpuslist";
    public final static String THESAURUSLIST_PARAMNAME = "thesauruslist";
    private FicheTableParameters ficheTableParameters;
    private String newTableExportName;
    private Corpus[] corpusArray;
    private Thesaurus[] thesaurusArray;

    public TableExportCreationCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() {
        EditOrigin editOrigin = bdfUser.newEditOrigin(Domains.EXPORTATION + "/" + COMMANDNAME);
        TableExportManager manager = bdfServer.getTableExportManager();
        TableExportDescription tableExportDescription = null;
        if (corpusArray != null) {
            for (Corpus corpus : corpusArray) {
                TableDef tableDef = FicheTableDefBuilder.fromUi(bdfServer, corpus, ficheTableParameters, getPermissionSummary());
                tableExportDescription = manager.putTableDef(newTableExportName, corpus.getSubsetKey(), tableDef, editOrigin);
            }
        }
        if (thesaurusArray != null) {
            for (Thesaurus thesaurus : thesaurusArray) {
                TableDef tableDef = MotcleTableDefBuilder.fromThesaurusMetadata(thesaurus, bdfServer.getThesaurusLangChecker());
                tableExportDescription = manager.putTableDef(newTableExportName, thesaurus.getSubsetKey(), tableDef, editOrigin);
            }
        }
        if (tableExportDescription != null) {
            putResultObject(TABLEEXPORTDESCRIPTION_OBJ, tableExportDescription);
            setDone("_ done.exportation.tableexportcreation", newTableExportName);
        }
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        BdfInstructionUtils.checkGlobalAdminPermission(bdfServer, bdfUser);
        newTableExportName = getMandatory(NEWTABLEEXPORT_PARAMNAME).trim();
        if (newTableExportName.length() == 0) {
            setError("_ error.empty.exportname");
            return false;
        }
        if (!StringUtils.isTechnicalName(newTableExportName, true)) {
            setError("_ error.wrong.exportname", newTableExportName);
            return false;
        }
        if (bdfServer.getTableExportManager().containsTableExport(newTableExportName)) {
            setError("_ error.existing.tableexport", newTableExportName);
            return false;
        }
        try {
            ficheTableParameters = FicheTableParametersParser.parse(requestMap);
        } catch (ErrorMessageException errorMessageException) {
            setCommandMessage(errorMessageException.getErrorMessage());
            return false;
        }
        String[] corpusNameArray = requestMap.getParameterValues(CORPUSLIST_PARAMNAME);
        if (corpusNameArray != null) {
            Set<Corpus> set = new LinkedHashSet<Corpus>();
            for (String corpusName : corpusNameArray) {
                SubsetKey corpusKey;
                try {
                    corpusKey = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, corpusName);
                    Corpus corpus = (Corpus) fichotheque.getSubset(corpusKey);
                    if (corpus != null) {
                        set.add(corpus);
                    }
                } catch (ParseException pe) {
                }
            }
            this.corpusArray = set.toArray(new Corpus[set.size()]);
        }
        String[] thesaurusNameArray = requestMap.getParameterValues(THESAURUSLIST_PARAMNAME);
        if (thesaurusNameArray != null) {
            Set<Thesaurus> set = new LinkedHashSet<Thesaurus>();
            for (String thesaurusName : thesaurusNameArray) {
                SubsetKey thesaurusKey;
                try {
                    thesaurusKey = SubsetKey.parse(SubsetKey.CATEGORY_THESAURUS, thesaurusName);
                    Thesaurus thesaurus = (Thesaurus) fichotheque.getSubset(thesaurusKey);
                    if (thesaurus != null) {
                        set.add(thesaurus);
                    }
                } catch (ParseException pe) {
                }
            }
            this.thesaurusArray = set.toArray(new Thesaurus[set.size()]);
        }
        if ((thesaurusNameArray == null) && ((corpusNameArray == null))) {
            this.corpusArray = FichothequeUtils.toCorpusArray(fichotheque, EligibilityUtils.ALL_SUBSET_PREDICATE);
        }
        return true;
    }

}
