/* BdfServer_Email - Copyright (c) 2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.email.datasource;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.exportation.transformation.TransformationConstants;
import fr.exemole.bdfserver.api.managers.TransformationManager;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.exportation.transformation.BdfTransformationUtils;
import fr.exemole.bdfserver.tools.users.BdfUserUtils;
import java.text.ParseException;
import java.util.Map;
import javax.activation.DataSource;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.Fiches;
import net.fichotheque.exportation.transformation.StreamTemplate;
import net.fichotheque.exportation.transformation.TemplateKey;
import net.fichotheque.exportation.transformation.TransformationKey;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.extraction.ExtractionSource;
import net.fichotheque.extraction.def.ExtractionDef;
import net.fichotheque.tools.extraction.builders.BuilderUtils;
import net.fichotheque.xml.extraction.ExtractionXMLPart;
import net.mapeadores.util.text.ValidExtension;


/**
 *
 * @author Vincent Calame
 */
public class StreamDataSourceFactory {

    private StreamDataSourceFactory() {

    }

    public static DataSource newFicheInstance(BdfServer bdfServer, BdfUser bdfUser, PermissionSummary permissionSummary, FicheMeta ficheMeta, ValidExtension extension, String templateName) {
        SubsetKey corpusKey = ficheMeta.getCorpus().getSubsetKey();
        String name = corpusKey.getSubsetName() + "-" + String.valueOf(ficheMeta.getId()) + "." + extension.toString();
        TemplateKey templateKey = getTemplateKey(new TransformationKey(corpusKey), extension, templateName, bdfUser);
        ExtractionContext extractionContext = BdfServerUtils.initExtractionContextBuilder(bdfServer, bdfUser)
                .setSubsetAccessPredicate(permissionSummary.getSubsetAccessPredicate())
                .toExtractionContext();
        StreamTemplate streamTemplate = getStreamTemplate(bdfServer, templateKey);
        if (streamTemplate instanceof StreamTemplate.Xslt) {
            return initXslt((StreamTemplate.Xslt) streamTemplate, bdfServer, bdfUser, templateKey, name, extractionContext, ficheMeta);
        } else {
            throw new IllegalStateException("unknown streamTemplate Type");
        }
    }

    public static DataSource newCompilationInstance(BdfServer bdfServer, BdfUser bdfUser, PermissionSummary permissionSummary, Fiches fiches, ValidExtension extension, String templateName) {
        String name = "compilation" + "." + extension.toString();
        ExtractionContext extractionContext = BdfServerUtils.initExtractionContextBuilder(bdfServer, bdfUser)
                .setSubsetAccessPredicate(permissionSummary.getSubsetAccessPredicate())
                .setSelectedFiches(fiches)
                .toExtractionContext();
        TemplateKey templateKey = getTemplateKey(TransformationKey.COMPILATION_TRANSFORMATIONKEY, extension, templateName, bdfUser);
        StreamTemplate streamTemplate = getStreamTemplate(bdfServer, templateKey);
        if (streamTemplate instanceof StreamTemplate.Xslt) {
            return initXslt((StreamTemplate.Xslt) streamTemplate, bdfServer, bdfUser, templateKey, name, extractionContext, null);
        } else {
            throw new IllegalStateException("unknown streamTemplate Type");
        }
    }

    private static TemplateKey getTemplateKey(TransformationKey transformationKey, ValidExtension extension, String templateName, BdfUser bdfUser) {
        if (templateName == null) {
            templateName = BdfUserUtils.getStreamTemplateName(bdfUser, transformationKey, extension.toString());
        }
        TemplateKey templateKey;
        try {
            templateKey = TemplateKey.parse(transformationKey, extension, templateName);
        } catch (ParseException pe) {
            templateKey = TemplateKey.toDefault(transformationKey, extension);
        }
        return templateKey;
    }

    private static StreamTemplate getStreamTemplate(BdfServer bdfServer, TemplateKey templateKey) {
        TransformationManager transformationManager = bdfServer.getTransformationManager();
        StreamTemplate streamTemplate = transformationManager.getStreamTemplate(templateKey);
        if ((streamTemplate == null) && (!templateKey.isDefault())) {
            streamTemplate = transformationManager.getStreamTemplate(templateKey.toDefaultTemplateKey());
        }
        if (streamTemplate == null) {
            throw new IllegalStateException("Bad streamTemplateName and no default: " + templateKey.toString());
        }
        return streamTemplate;
    }

    private static DataSource initXslt(StreamTemplate.Xslt streamTemplate, BdfServer bdfServer, BdfUser bdfUser, TemplateKey templateKey, String name, ExtractionContext extractionContext, Object dynamicObject) {
        ExtractionDef extractionDef = streamTemplate.getCustomExtractionDef();
        if (extractionDef == null) {
            extractionDef = BdfTransformationUtils.getDefaultExtractionDef(bdfServer, templateKey.getTransformationKey());
        }
        ExtractionSource extractionSource = BuilderUtils.getExtractionSource(dynamicObject, extractionContext, extractionDef);
        String extractionString = ExtractionXMLPart.toString(extractionContext, extractionDef, extractionSource, false);
        Map<String, Object> paramMap = BdfTransformationUtils.newParamMap(bdfServer, bdfUser);
        paramMap.put(TransformationConstants.INCLUDESCRIPTS_PARAMETER, "");
        return new XsltStreamDataSource(name, streamTemplate, extractionString, paramMap);
    }

}
