/* FichothequeLib_Tools - Copyright (c) 2010-2018 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.extraction.builders;

import java.util.List;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.extraction.ExtractionConstants;
import net.fichotheque.extraction.def.ExtraitAddendaDef;
import net.fichotheque.extraction.def.ExtraitAlbumDef;
import net.fichotheque.extraction.def.ExtraitCorpusDef;
import net.fichotheque.extraction.def.ExtraitThesaurusDef;
import net.fichotheque.extraction.def.FicheFilter;
import net.fichotheque.extraction.filterunit.CorpsdeficheFilterUnit;
import net.fichotheque.extraction.filterunit.FieldKeyFilterUnit;
import net.fichotheque.extraction.filterunit.FieldNamePrefixFilterUnit;
import net.fichotheque.extraction.filterunit.FilterUnit;


/**
 *
 * @author Vincent Calame
 */
public class FicheFilterBuilder {

    public final static FicheFilter TITRE_FICHEFILTER = new TitreFicheFilter();
    private String tagName = ExtractionConstants.FICHE_DEFAULT_TAGNAME;
    private FilterUnit[] filterUnitArray = null;
    private boolean withCorpsdefiche = false;

    public FicheFilterBuilder() {
    }

    public FicheFilterBuilder setTagName(String tagName) {
        this.tagName = BuilderUtils.testTagName(tagName, ExtractionConstants.FICHE_DEFAULT_TAGNAME);
        return this;
    }

    public FicheFilterBuilder updateFilterUnitList(List<FilterUnit> filterUnitList) {
        if (filterUnitList == null) {
            filterUnitArray = null;
        }
        filterUnitArray = filterUnitList.toArray(new FilterUnit[filterUnitList.size()]);
        for (FilterUnit filterUnit : filterUnitArray) {
            if (filterUnit instanceof CorpsdeficheFilterUnit) {
                withCorpsdefiche = true;
            } else if (filterUnit instanceof FieldKeyFilterUnit) {
                FieldKey fieldKey = ((FieldKeyFilterUnit) filterUnit).getFieldKey();
                if (fieldKey.ownsToCorpsdefiche()) {
                    withCorpsdefiche = true;
                }
            } else if (filterUnit instanceof FieldNamePrefixFilterUnit) {
                short category = ((FieldNamePrefixFilterUnit) filterUnit).getCategory();
                if (category == FieldKey.SECTION_CATEGORY) {
                    withCorpsdefiche = true;
                }
            }
        }
        return this;
    }

    public FicheFilter toFicheFilter() {
        List<FilterUnit> filterUnitList;
        if (filterUnitArray == null) {
            filterUnitList = FilterUnitUtils.EMPTY_FILTERUNITLIST;
        } else {
            filterUnitList = FilterUnitUtils.wrap(filterUnitArray);
        }
        return new InternalFicheFilter(tagName, filterUnitList, withCorpsdefiche);
    }

    public static FicheFilterBuilder init() {
        return new FicheFilterBuilder();
    }

    public static FicheFilter newFicheInstance(String tagName) {
        FicheFilter liageFicheFilter = TITRE_FICHEFILTER;
        ExtraitCorpusDef extraitCorpusDef = BuilderUtils.getExtraitCorpusDef(liageFicheFilter, IntituleClauseBuilder.DEFAULT);
        ExtraitThesaurusDef extraitThesaurusDef = BuilderUtils.getExtraitThesaurusDef(MotcleFilterBuilder.init().setWithLabels(true).toMotcleFilter());
        ExtraitAlbumDef extraitAlbumDef = BuilderUtils.getExtraitAlbumDef(IllustrationFilterBuilder.init().toIllustrationFilter());
        ExtraitAddendaDef extraitAddendaDef = BuilderUtils.getExtraitAddendaDef(DocumentFilterBuilder.init().toDocumentFilter());
        List<FilterUnit> filterUnitList = FilterUnitListBuilder.init()
                .addEnteteFilter()
                .addCorpsdeficheFilter()
                .addExtraitAddendaFilter(extraitAddendaDef)
                .addExtraitThesaurusFilter(extraitThesaurusDef)
                .addExtraitCorpusFilter(extraitCorpusDef)
                .addExtraitAlbumFilter(extraitAlbumDef)
                .addChronoFilter()
                .addFieldKeyFilter(FieldKey.LANG, null)
                .addIntituleFilter()
                .toFilterUnitList();
        return FicheFilterBuilder.init()
                .setTagName(tagName)
                .updateFilterUnitList(filterUnitList)
                .toFicheFilter();
    }

    public static FicheFilter fromFilterUnitList(List<FilterUnit> filterUnitList) {
        return FicheFilterBuilder.init()
                .updateFilterUnitList(filterUnitList)
                .toFicheFilter();
    }


    private static class InternalFicheFilter implements FicheFilter {

        private final String tagName;
        private final List<FilterUnit> filterUnitList;
        private final boolean withCorpsdefiche;


        private InternalFicheFilter(String tagName, List<FilterUnit> filterUnitList, boolean withCorpsdefiche) {
            this.tagName = tagName;
            this.filterUnitList = filterUnitList;
            this.withCorpsdefiche = withCorpsdefiche;
        }

        @Override
        public String getTagName() {
            return tagName;
        }

        @Override
        public boolean isWithCorpsdefiche() {
            return withCorpsdefiche;
        }

        @Override
        public List<FilterUnit> getFilterUnitList() {
            return filterUnitList;
        }

    }


    private static class TitreFicheFilter implements FicheFilter {

        private final List<FilterUnit> filterUnitList;

        private TitreFicheFilter() {
            FilterUnit[] array = new FilterUnit[2];
            array[0] = FilterUnitUtils.toFieldKeyFilterUnit(FieldKey.TITRE, null);
            array[1] = FilterUnitUtils.INTITULE_FILTERUNIT;
            filterUnitList = FilterUnitUtils.wrap(array);
        }

        @Override
        public boolean isWithCorpsdefiche() {
            return false;
        }

        @Override
        public List<FilterUnit> getFilterUnitList() {
            return filterUnitList;
        }

        @Override
        public String getTagName() {
            return ExtractionConstants.FICHE_DEFAULT_TAGNAME;
        }

    }

}
