/* FichothequeLib_Xml - Copyright (c) 2009-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.xml.extraction;

import java.io.IOException;
import java.text.ParseException;
import net.fichotheque.Fichotheque;
import net.fichotheque.Subset;
import net.fichotheque.SubsetKey;
import net.fichotheque.addenda.Addenda;
import net.fichotheque.addenda.Document;
import net.fichotheque.addenda.Version;
import net.fichotheque.album.Album;
import net.fichotheque.album.Illustration;
import net.fichotheque.album.metadata.AlbumDim;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.fiche.Atts;
import net.fichotheque.corpus.fiche.Cdatadiv;
import net.fichotheque.corpus.fiche.Code;
import net.fichotheque.corpus.fiche.Div;
import net.fichotheque.corpus.fiche.FicheBlock;
import net.fichotheque.corpus.fiche.FicheBlocks;
import net.fichotheque.corpus.fiche.H;
import net.fichotheque.corpus.fiche.Insert;
import net.fichotheque.corpus.fiche.Li;
import net.fichotheque.corpus.fiche.Ln;
import net.fichotheque.corpus.fiche.P;
import net.fichotheque.corpus.fiche.S;
import net.fichotheque.corpus.fiche.Table;
import net.fichotheque.corpus.fiche.Td;
import net.fichotheque.corpus.fiche.TextContent;
import net.fichotheque.corpus.fiche.Tr;
import net.fichotheque.corpus.fiche.Ul;
import net.fichotheque.corpus.fiche.ZoneBlock;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.extraction.def.MotcleFilter;
import net.fichotheque.metadata.FichothequeMetadata;
import net.fichotheque.namespaces.BdfSpace;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.utils.AlbumUtils;
import net.fichotheque.utils.FichothequeUtils;
import net.fichotheque.utils.ThesaurusUtils;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.awt.ResizeInfo;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.XMLPart;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class FicheBlockXMLPart extends XMLPart {

    private final MotcleXMLPart motcleXMLPart;
    private final ExtractionContext extractionContext;
    private final String imageBaseUrl;
    private final String audioBaseUrl;
    private Corpus defaultCorpus;

    public FicheBlockXMLPart(XMLWriter xmlWriter, ExtractionContext extractionContext) {
        super(xmlWriter);
        this.extractionContext = extractionContext;
        FichothequeMetadata fichothequeMetadata = extractionContext.getFichotheque().getFichothequeMetadata();
        Attribute imageAttribute = fichothequeMetadata.getAttributes().getAttribute(BdfSpace.IMAGEBASEURL_KEY);
        if (imageAttribute != null) {
            this.imageBaseUrl = imageAttribute.getFirstValue();
        } else {
            this.imageBaseUrl = null;
        }
        Attribute audioAttribute = fichothequeMetadata.getAttributes().getAttribute(BdfSpace.AUDIOBASEURL_KEY);
        if (audioAttribute != null) {
            this.audioBaseUrl = audioAttribute.getFirstValue();
        } else {
            this.audioBaseUrl = null;
        }
        this.motcleXMLPart = new MotcleXMLPart(xmlWriter, extractionContext);
    }

    public void setDefaultCorpus(Corpus defaultCorpus) {
        this.defaultCorpus = defaultCorpus;
    }

    public void addFicheBlock(FicheBlock ficheBlock) throws IOException {
        if (ficheBlock instanceof P) {
            addP((P) ficheBlock);
        } else if (ficheBlock instanceof H) {
            addH((H) ficheBlock);
        } else if (ficheBlock instanceof Ul) {
            addUl((Ul) ficheBlock);
        } else if (ficheBlock instanceof Code) {
            addCode((Code) ficheBlock);
        } else if (ficheBlock instanceof Table) {
            addTable((Table) ficheBlock);
        } else if (ficheBlock instanceof Insert) {
            addInsert((Insert) ficheBlock);
        } else if (ficheBlock instanceof Div) {
            addDiv((Div) ficheBlock);
        } else if (ficheBlock instanceof Cdatadiv) {
            addCdataDiv((Cdatadiv) ficheBlock);
        } else {
            throw new IllegalArgumentException("unknown FicheBlock Implementation");
        }
    }

    public void addFicheBlocks(FicheBlocks ficheBlocks) throws IOException {
        int size = ficheBlocks.size();
        for (int i = 0; i < size; i++) {
            addFicheBlock(ficheBlocks.get(i));
        }
    }

    public void addTextContent(TextContent content) throws IOException {
        for (Object obj : content) {
            if (obj instanceof String) {
                addText((String) obj);
            } else if (obj instanceof S) {
                addS((S) obj);
            }
        }
    }

    public void addP(P p) throws IOException {
        startOpenTag("p");
        addAttribute("type", P.typeToString(p.getType()));
        addAttribute("source", p.getSource());
        addAtts(p);
        endOpenTag();
        addTextContent(p);
        closeTag("p", false);
    }

    public void addH(H h) throws IOException {
        startOpenTag("h");
        addAttribute("level", String.valueOf(h.getLevel()));
        addAtts(h);
        endOpenTag();
        addTextContent(h);
        closeTag("h", false);
    }

    public void addUl(Ul ul) throws IOException {
        startOpenTag("ul");
        addAtts(ul);
        endOpenTag();
        for (Li li : ul) {
            addLi(li);
        }
        closeTag("ul");
    }

    public void addLi(Li li) throws IOException {
        startOpenTag("li");
        addAtts(li.getAtts());
        endOpenTag();
        addFicheBlocks(li);
        closeTag("li");
    }

    public void addCode(Code code) throws IOException {
        startOpenTag("code");
        addAttribute("type", Code.typeToString(code.getType()));
        addAtts(code);
        endOpenTag();
        addZoneBlockElements(code);
        for (Ln ln : code) {
            addLn(ln);
        }
        closeTag("code");
    }

    public void addLn(Ln ln) throws IOException {
        int indentation = ln.getIndentation();
        startOpenTag("ln");
        if (indentation > 0) {
            addAttribute("indent", String.valueOf(indentation));
        }
        addAtts(ln.getAtts());
        endOpenTag();
        addText(ln.getValue());
        closeTag("ln", false);
    }

    public void addDiv(Div div) throws IOException {
        startOpenTag("div");
        XMLUtils.addXmlLangAttribute(this, div.getLang());
        addAtts(div);
        endOpenTag();
        addZoneBlockElements(div);
        openTag("fbl");
        addFicheBlocks(div);
        closeTag("fbl");
        closeTag("div");
    }

    public void addCdataDiv(Cdatadiv cdatadiv) throws IOException {
        startOpenTag("cdatadiv");
        addAtts(cdatadiv);
        endOpenTag();
        addZoneBlockElements(cdatadiv);
        startOpenTag("cdata");
        endOpenTag();
        addCData(cdatadiv.getCdata());
        closeTag("cdata", false);
        closeTag("cdatadiv");
    }

    public void addS(S s) throws IOException {
        short sType = s.getType();
        startOpenTag("s", false);
        addAttribute("type", S.typeToString(sType));
        String ref = s.getRef();
        addAttribute("ref", ref);
        if ((sType == S.IMAGE) && (ref.length() > 0) && (imageBaseUrl != null)) {
            if (!StringUtils.isAbsoluteUrlString(ref)) {
                addAttribute("base", imageBaseUrl);
            }
        }
        addAtts(s.getAtts());
        if (!S.isSpecialRef(sType)) {
            if (extractionContext.getLinkAnalyser().isExternalLink(ref)) {
                addAttribute("link", "external");
            }
        }
        String mode = s.getMode();
        if (!mode.isEmpty()) {
            String cdata = extractionContext.getModeResolver().toCData(mode, s.getValue());
            if (!cdata.isEmpty()) {
                addAttribute("cdata", cdata);
            }
        }
        switch (sType) {
            case S.FICHE:
                closeFicheS(s);
                break;
            case S.MOTCLE:
                closeMotcleS(s);
                break;
            default: {
                closeDefaultS(s);
            }
        }
    }

    private void closeMotcleS(S s) throws IOException {
        String ref = checkFragment(s);
        Fichotheque fichotheque = extractionContext.getFichotheque();
        Motcle motcle = null;
        try {

            motcle = ThesaurusUtils.parseGlobalIdalpha(ref, fichotheque);
        } catch (ParseException pe) {
            try {
                motcle = (Motcle) FichothequeUtils.parseGlobalId(ref, fichotheque, SubsetKey.CATEGORY_THESAURUS, null);
            } catch (ParseException pe2) {
                motcle = null;
            }
        }
        if (motcle == null) {
            addAttribute("ref-error", ref);
            closeDefaultS(s);
            return;
        }
        endOpenTag();
        MotcleFilter motcleFilter;
        String value = s.getValue();
        if (value.length() > 0) {
            addSimpleElement("value", value);
            motcleFilter = ExtractionXMLUtils.NONE_MOTCLEFILTER;
        } else {
            motcleFilter = ExtractionXMLUtils.LABELS_MOTCLEFILTER;
        }
        motcleXMLPart.addMotcle(ExtractionXMLUtils.toMotcleExtractInfo(motcle, motcleFilter));
        closeTag("s", false);
    }

    private void closeFicheS(S s) throws IOException {
        String ref = checkFragment(s);
        FicheMeta other;
        try {
            other = (FicheMeta) FichothequeUtils.parseGlobalId(ref, extractionContext.getFichotheque(), SubsetKey.CATEGORY_CORPUS, defaultCorpus);
        } catch (ParseException pe) {
            other = null;
        }
        if (other == null) {
            addAttribute("ref-error", ref);
            closeDefaultS(s);
            return;
        }
        Lang otherLang = other.getLang();
        if (otherLang != null) {
            addAttribute("lang", otherLang.toString());
        }
        addAttribute("corpus", other.getCorpus().getSubsetKey().getSubsetName());
        addAttribute("id", other.getId());
        Subset master = other.getCorpus().getMasterSubset();
        if ((master != null) && (master instanceof Thesaurus)) {
            String idalpha = ((Motcle) master.getSubsetItemById(other.getId())).getIdalpha();
            if (idalpha != null) {
                addAttribute("idalpha", idalpha);
            }
        }
        endOpenTag();
        String value = s.getValue();
        if (value.length() > 0) {
            addText(value);
        } else {
            addText(other.getTitre());
        }
        closeTag("s", false);
    }

    private void closeDefaultS(S s) throws IOException {
        String value = s.getValue();
        if (value.length() > 0) {
            endOpenTag();
            addText(value);
            closeTag("s", false);
        } else {
            closeEmptyTag();
        }
    }

    private String checkFragment(S s) throws IOException {
        String ref = s.getRef();
        int idx = ref.indexOf('#');
        if (idx > 0) {
            addAttribute("fragment", ref.substring(idx + 1));
            ref = ref.substring(0, idx);
        }
        return ref;
    }


    public void addTable(Table table) throws IOException {
        startOpenTag("table");
        addAtts(table);
        addAttribute("cols", Table.checkColCount(table));
        endOpenTag();
        addZoneBlockElements(table);
        for (Tr tr : table) {
            addTr(tr);
        }
        closeTag("table");
    }

    public void addTr(Tr tr) throws IOException {
        startOpenTag("tr");
        addAtts(tr.getAtts());
        endOpenTag();
        for (Td td : tr) {
            addTd(td);
        }
        closeTag("tr");
    }

    public void addTd(Td td) throws IOException {
        startOpenTag("td");
        addAttribute("type", Td.typeToString(td.getType()));
        if (td.isNumber()) {
            addAttribute("format", "number");
        }
        addAtts(td.getAtts());
        endOpenTag();
        addTextContent(td);
        closeTag("td", false);
    }

    public void addInsert(Insert insert) throws IOException {
        startOpenTag("insert");
        addAttribute("type", Insert.typeToString(insert.getType()));
        String src = insert.getSrc();
        String ref = insert.getRef();
        addAttribute("src", src);
        if ((!src.isEmpty()) && (!StringUtils.isAbsoluteUrlString(src))) {
            if (Insert.isDocumentType(insert.getType())) {
                if (audioBaseUrl != null) {
                    addAttribute("base", audioBaseUrl);
                }
            } else {
                if (imageBaseUrl != null) {
                    addAttribute("base", imageBaseUrl);
                }
            }
        }
        addAttribute("ref", ref);
        if ((!ref.isEmpty()) && (extractionContext.getLinkAnalyser().isExternalLink(ref))) {
            addAttribute("link", "external");
        }
        addAttribute("position", Insert.positionToString(insert.getPosition()));
        int width = insert.getWidth();
        if (width >= 0) {
            addAttribute("width", String.valueOf(width));
        }
        int height = insert.getHeight();
        if (height >= 0) {
            addAttribute("height", String.valueOf(height));
        }
        SubsetKey subsetKey = insert.getSubsetKey();
        Document insertDocument = null;
        if (subsetKey != null) {
            int id = insert.getId();
            if (subsetKey.isAddendaSubset()) {
                addAttribute("addenda", subsetKey.getSubsetName());
                addAttribute("id", id);
                Addenda addenda = (Addenda) extractionContext.getFichotheque().getSubset(subsetKey);
                if (addenda != null) {
                    insertDocument = addenda.getDocumentById(id);
                    if (insertDocument != null) {
                        addAttribute("basename", insertDocument.getBasename());
                    }
                }
            } else {
                String albumDimName = insert.getAlbumDimName();
                addAttribute("album", subsetKey.getSubsetName());
                addAttribute("id", id);
                addAttribute("albumdim", albumDimName);
                boolean done = false;
                Album album = (Album) extractionContext.getFichotheque().getSubset(subsetKey);
                if (album != null) {
                    Illustration illustration = album.getIllustrationById(id);
                    if (illustration != null) {
                        addAttribute("format", AlbumUtils.formatTypeToString(illustration.getFormatType()));
                        int albumWidth = illustration.getOriginalWidth();
                        int albumHeight = illustration.getOriginalHeight();
                        if (albumDimName != null) {
                            AlbumDim albumDim = album.getAlbumMetadata().getAlbumDimByName(albumDimName);
                            if (albumDim != null) {
                                ResizeInfo.Check check = albumDim.getResizeInfo().checkDim(albumWidth, albumHeight);
                                albumWidth = check.getWidth();
                                albumHeight = check.getHeight();
                            }
                        }
                        addAttribute("album-width", albumWidth);
                        addAttribute("album-height", albumHeight);
                        addAttribute("album-ratio", String.valueOf(((float) albumWidth) / albumHeight));
                        done = true;
                    }
                }
                if (!done) {
                    addAttribute("format", "png");
                }
            }
        }
        if ((width > 0) && (height > 0)) {
            addAttribute("ratio", String.valueOf(((float) width) / height));
        }
        addAtts(insert);
        endOpenTag();
        if (insertDocument != null) {
            for (Version version : insertDocument.getVersionList()) {
                startOpenTag("insertversion");
                addAttribute("extension", version.getExtension());
                addAttribute("type", extractionContext.getMimeTypeResolver().getMimeType(version.getExtension()));
                closeEmptyTag();
            }
        }
        addZoneBlockElements(insert);
        addZoneBlockElement("alt", insert.getAlt());
        addZoneBlockElement("credit", insert.getCredit());
        closeTag("insert");
    }

    private void addZoneBlockElements(ZoneBlock zoneBlock) throws IOException {
        addZoneBlockElement("numero", zoneBlock.getNumero());
        addZoneBlockElement("legende", zoneBlock.getLegende());
    }

    private void addZoneBlockElement(String name, TextContent textContent) throws IOException {
        if (textContent.size() > 0) {
            startOpenTag(name);
            endOpenTag();
            addTextContent(textContent);
            closeTag(name, false);
        }
    }

    private void addAtts(FicheBlock ficheBlock) throws IOException {
        addAtts(ficheBlock.getAtts());
    }

    private void addAtts(Atts atts) throws IOException {
        int attLength = atts.size();
        for (int i = 0; i < attLength; i++) {
            addAttribute("att-" + atts.getName(i), atts.getValue(i));
        }
    }


}
