/* FichothequeLib_Xml - Copyright (c) 2006-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.xml.extraction;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Predicate;
import net.fichotheque.Fichotheque;
import net.fichotheque.Subset;
import net.fichotheque.SubsetKey;
import net.fichotheque.addenda.Addenda;
import net.fichotheque.addenda.Document;
import net.fichotheque.album.Album;
import net.fichotheque.album.Illustration;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.fiche.Datation;
import net.fichotheque.corpus.fiche.FicheBlocks;
import net.fichotheque.corpus.fiche.FicheItem;
import net.fichotheque.corpus.fiche.FicheItems;
import net.fichotheque.corpus.fiche.Langue;
import net.fichotheque.corpus.fiche.Para;
import net.fichotheque.corpus.metadata.CorpusField;
import net.fichotheque.corpus.metadata.CorpusMetadata;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.corpus.metadata.FieldOptionsConstants;
import net.fichotheque.croisement.Croisement;
import net.fichotheque.croisement.Croisements;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.extraction.def.ExtraitAddendaDef;
import net.fichotheque.extraction.def.ExtraitAlbumDef;
import net.fichotheque.extraction.def.ExtraitCorpusDef;
import net.fichotheque.extraction.def.ExtraitThesaurusDef;
import net.fichotheque.extraction.def.FicheFilter;
import net.fichotheque.extraction.def.FicheRattachementDef;
import net.fichotheque.extraction.def.MotcleRattachementDef;
import net.fichotheque.extraction.filterunit.ChronoFilterUnit;
import net.fichotheque.extraction.filterunit.CorpsdeficheFilterUnit;
import net.fichotheque.extraction.filterunit.EnteteFilterUnit;
import net.fichotheque.extraction.filterunit.ExtraitAddendaFilterUnit;
import net.fichotheque.extraction.filterunit.ExtraitAlbumFilterUnit;
import net.fichotheque.extraction.filterunit.ExtraitCorpusFilterUnit;
import net.fichotheque.extraction.filterunit.ExtraitThesaurusFilterUnit;
import net.fichotheque.extraction.filterunit.FicheRattachementFilterUnit;
import net.fichotheque.extraction.filterunit.FieldKeyFilterUnit;
import net.fichotheque.extraction.filterunit.FieldNamePrefixFilterUnit;
import net.fichotheque.extraction.filterunit.FilterUnit;
import net.fichotheque.extraction.filterunit.IntituleFilterUnit;
import net.fichotheque.extraction.filterunit.LotFilterUnit;
import net.fichotheque.extraction.filterunit.MotcleRattachementFilterUnit;
import net.fichotheque.extraction.run.AddendaExtractor;
import net.fichotheque.extraction.run.AlbumExtractor;
import net.fichotheque.extraction.run.CorpusExtractor;
import net.fichotheque.extraction.run.ExtraitCorpusResult;
import net.fichotheque.extraction.run.FicheExtractInfo;
import net.fichotheque.extraction.run.ThesaurusExtractor;
import net.fichotheque.namespaces.BdfXmlSpace;
import net.fichotheque.pointeurs.FichePointeur;
import net.fichotheque.pointeurs.SubsetItemPointeur;
import net.fichotheque.selection.DocumentSelector;
import net.fichotheque.selection.FicheSelector;
import net.fichotheque.selection.IllustrationSelector;
import net.fichotheque.selection.MotcleSelector;
import net.fichotheque.selection.SelectionContext;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.utils.EligibilityUtils;
import net.fichotheque.utils.FichothequeUtils;
import net.fichotheque.utils.PointeurUtils;
import net.fichotheque.utils.pointeurs.PointeurFactory;
import net.fichotheque.utils.selection.DocumentSelectorBuilder;
import net.fichotheque.utils.selection.FicheSelectorBuilder;
import net.fichotheque.utils.selection.IllustrationSelectorBuilder;
import net.fichotheque.utils.selection.MotcleSelectorBuilder;
import net.fichotheque.utils.selection.SelectionContextBuilder;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.Attributes;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.primitives.FuzzyDate;
import net.mapeadores.util.text.alphabet.AlphabeticEntry;
import net.mapeadores.util.xml.FillControllerXMLPart;
import net.mapeadores.util.xml.XMLPart;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class FicheXMLPart extends XMLPart {

    private final FilterUnitXMLPart filterUnitPart;
    private FicheFilter ficheFilter;
    private FichePointeur fichePointeur;

    public FicheXMLPart(XMLWriter xmlWriter, ExtractionContext extractionContext) {
        super(xmlWriter);
        filterUnitPart = new FilterUnitXMLPart(xmlWriter, extractionContext);
    }

    public void addFiche(FicheExtractInfo ficheExtractInfo) throws IOException {
        updatePointeur(ficheExtractInfo);
        ficheFilter = ficheExtractInfo.getFicheFilter();
        FicheMeta ficheMeta = ficheExtractInfo.getFicheMeta();
        Croisement croisement = ficheExtractInfo.getCroisement();
        String tagName = ficheFilter.getTagName();
        if (tagName != null) {
            startOpenTag(tagName);
            addAttribute("id", String.valueOf(ficheMeta.getId()));
            XMLUtils.addXmlLangAttribute(this, ficheMeta.getLang());
            int poids = ExtractionXMLUtils.getMainPoids(croisement);
            if (poids > 0) {
                addAttribute("poids", String.valueOf(poids));
            }
            addAttribute("corpus", ficheMeta.getCorpus().getSubsetKey().getSubsetName());
            if (ficheFilter.getFilterUnitList().isEmpty()) {
                closeEmptyTag();
                return;
            }
            endOpenTag();
            ExtractionXMLUtils.writeCroisement(this, croisement);
        }
        addGroupClauseObject(ficheExtractInfo.getGroupClauseObject());
        filterUnitPart.addFilterUnitList(ficheFilter.getFilterUnitList());
        if (tagName != null) {
            closeTag(tagName);
        }
    }

    private void updatePointeur(FicheExtractInfo ficheExtractInfo) {
        boolean ficheComplete = (ficheExtractInfo.getFicheFilter().isWithCorpsdefiche());
        FicheMeta ficheMeta = ficheExtractInfo.getFicheMeta();
        Corpus corpus = ficheMeta.getCorpus();
        if (fichePointeur != null) {
            if (!fichePointeur.getCorpus().equals(corpus)) {
                fichePointeur = null;
            } else if (fichePointeur.isFicheComplete() != ficheComplete) {
                fichePointeur = null;
            }
        }
        if (fichePointeur == null) {
            fichePointeur = PointeurFactory.newFichePointeur(corpus, ficheComplete);
        }
        fichePointeur.setCurrentSubsetItem(ficheMeta);
        filterUnitPart.updatePointeur(fichePointeur);
    }

    public void addGroupClauseObject(Object groupClauseObject) throws IOException {
        if (groupClauseObject == null) {
            return;
        }
        if (groupClauseObject instanceof AlphabeticEntry) {
            AlphabeticEntry alphabeticEntry = (AlphabeticEntry) groupClauseObject;
            startOpenTag("alphabet");
            if (alphabeticEntry.isWithArticle()) {
                addAttribute("article", alphabeticEntry.getArticleString());
                addAttribute("espace", (alphabeticEntry.isWithSeparationSpace()) ? 1 : 0);
            }
            addAttribute("titre", alphabeticEntry.getEntryString());
            closeEmptyTag();
        }
    }


    private static class FilterUnitXMLPart extends XMLPart {

        private final ExtractionContext extractionContext;
        private final FicheItemXMLPart ficheItemXMLPart;
        private final FicheBlockXMLPart ficheBlockXMLPart;
        private ExtraitCorpusXMLPart extraitCorpusXMLPart = null;
        private ExtraitThesaurusXMLPart extraitThesaurusXMLPart = null;
        private ExtraitAlbumXMLPart extraitAlbumXMLPart = null;
        private ExtraitAddendaXMLPart extraitAddendaXMLPart = null;
        private FichePointeur fichePointeur;

        FilterUnitXMLPart(XMLWriter xmlWriter, ExtractionContext extractionContext) {
            super(xmlWriter);
            this.extractionContext = extractionContext;
            ficheItemXMLPart = new FicheItemXMLPart(xmlWriter, extractionContext);
            ficheBlockXMLPart = new FicheBlockXMLPart(xmlWriter, extractionContext);
        }

        private void updatePointeur(FichePointeur fichePointeur) {
            this.fichePointeur = fichePointeur;
            ficheBlockXMLPart.setDefaultCorpus(fichePointeur.getCorpus());
        }

        private void addFilterUnitList(List<FilterUnit> filterUnitList) throws IOException {
            Lang defaultLang = LocalisationUtils.getDefaultLang(extractionContext.getLangContext());
            SelectionContext selectionContext = SelectionContextBuilder.init(extractionContext, defaultLang)
                    .setSubsetAccessPredicate(extractionContext.getSubsetAccessPredicate())
                    .toSelectionContext();
            for (FilterUnit filterUnit : filterUnitList) {
                if (filterUnit instanceof EnteteFilterUnit) {
                    addEntete();
                } else if (filterUnit instanceof CorpsdeficheFilterUnit) {
                    addCorpsdefiche();
                } else if (filterUnit instanceof FieldKeyFilterUnit) {
                    FieldKeyFilterUnit fieldKeyFilterUnit = (FieldKeyFilterUnit) filterUnit;
                    addFieldKey(fieldKeyFilterUnit.getFieldKey(), fieldKeyFilterUnit.getAttributes());
                } else if (filterUnit instanceof FieldNamePrefixFilterUnit) {
                    addPrefixedField((FieldNamePrefixFilterUnit) filterUnit);
                } else if (filterUnit instanceof LotFilterUnit) {
                    addLot((LotFilterUnit) filterUnit);
                } else if (filterUnit instanceof ChronoFilterUnit) {
                    addChrono();
                } else if (filterUnit instanceof IntituleFilterUnit) {
                    ExtractionXMLUtils.writeIntitule(this, fichePointeur.getCurrentSubsetItem(), "fiche", extractionContext);
                } else if (filterUnit instanceof ExtraitAddendaFilterUnit) {
                    addExtraitAddenda(((ExtraitAddendaFilterUnit) filterUnit).getExtraitAddendaDef(), selectionContext);
                } else if (filterUnit instanceof ExtraitCorpusFilterUnit) {
                    addExtraitCorpus(((ExtraitCorpusFilterUnit) filterUnit).getExtraitCorpusDef());
                } else if (filterUnit instanceof ExtraitThesaurusFilterUnit) {
                    addExtraitThesaurus(((ExtraitThesaurusFilterUnit) filterUnit).getExtraitThesaurusDef(), selectionContext);
                } else if (filterUnit instanceof ExtraitAlbumFilterUnit) {
                    addExtraitAlbum(((ExtraitAlbumFilterUnit) filterUnit).getExtraitAlbumDef(), selectionContext);
                } else if (filterUnit instanceof FicheRattachementFilterUnit) {
                    addFicheRattachement(((FicheRattachementFilterUnit) filterUnit).getFicheRattachementDef());
                } else if (filterUnit instanceof MotcleRattachementFilterUnit) {
                    addMotcleRattachement(((MotcleRattachementFilterUnit) filterUnit).getMotcleRattachementDef());
                }
            }
        }

        private void addLot(LotFilterUnit lotFilterUnit) throws IOException {
            LotXMLPart lotXMLPart = new LotXMLPart(this, lotFilterUnit);
            FilterUnitXMLPart child = new FilterUnitXMLPart(lotXMLPart, extractionContext);
            child.updatePointeur(fichePointeur);
            child.addFilterUnitList(lotFilterUnit.getFilterUnitList());
            lotXMLPart.end();
        }

        private void addEntete() throws IOException {
            CorpusMetadata corpusMetadata = getCorpusMetatdata();
            addTitre();
            addSoustitre();
            for (CorpusField corpusField : corpusMetadata.getProprieteList()) {
                addPropriete(corpusField.getFieldKey(), null, getFieldOptionObject(corpusField, null));
            }
            for (CorpusField corpusField : corpusMetadata.getInformationList()) {
                addInformation(corpusField.getFieldKey(), null, getFieldOptionObject(corpusField, null));
            }
            addRedacteurs();
        }

        private Object getFieldOptionObject(CorpusField corpusField, Attributes attributes) {
            switch (corpusField.getFicheItemType()) {
                case CorpusField.IMAGE_FIELD:
                case CorpusField.LINK_FIELD:
                    return corpusField.getStringOption(FieldOptionsConstants.BASEURL_OPTION);
                case CorpusField.DATATION_FIELD:
                    if (attributes != null) {
                        Attribute attribute = attributes.getAttribute(BdfXmlSpace.LABELTYPE_ATTRIBUTEKEY);
                        if (attribute != null) {
                            String value = attribute.getFirstValue();
                            try {
                                return typeToShort(value);
                            } catch (IllegalArgumentException iae) {
                            }
                        }
                    }
                    return null;
                case CorpusField.MONTANT_FIELD:
                    if (attributes != null) {
                        Attribute attribute = attributes.getAttribute(BdfXmlSpace.SUBUNIT_ATTRIBUTEKEY);
                        if (attribute != null) {
                            return attribute.getFirstValue();
                        }
                    }
                    return null;
                default:
                    return null;
            }
        }

        private void addCorpsdefiche() throws IOException {
            CorpusMetadata corpusMetadata = getCorpusMetatdata();
            for (CorpusField corpusField : corpusMetadata.getSectionList()) {
                addSection(corpusField.getFieldKey(), null);
            }
        }

        private void addFieldKey(FieldKey fieldKey, Attributes attributes) throws IOException {
            CorpusMetadata corpusMetadata = getCorpusMetatdata();
            CorpusField corpusField = corpusMetadata.getCorpusField(fieldKey);
            if (corpusField == null) {
                return;
            }
            if (fieldKey.equals(FieldKey.TITRE)) {
                addTitre();
            } else if (fieldKey.equals(FieldKey.SOUSTITRE)) {
                addSoustitre();
            } else if (fieldKey.equals(FieldKey.REDACTEURS)) {
                addRedacteurs();
            } else if (fieldKey.equals(FieldKey.LANG)) {
                addLang();
            } else if (fieldKey.isProprieteFieldKey()) {
                addPropriete(fieldKey, null, getFieldOptionObject(corpusField, attributes));
            } else if (fieldKey.isInformationFieldKey()) {
                addInformation(fieldKey, null, getFieldOptionObject(corpusField, attributes));
            } else if (fieldKey.isSectionFieldKey()) {
                addSection(fieldKey, null);
            }
        }

        private void addPrefixedField(FieldNamePrefixFilterUnit filterUnit) throws IOException {
            Attributes attributes = filterUnit.getAttributes();
            String prefix = filterUnit.getPrefix();
            switch (filterUnit.getCategory()) {
                case FieldKey.PROPRIETE_CATEGORY:
                    addPrefixedPropriete(prefix, attributes);
                    break;
                case FieldKey.INFORMATION_CATEGORY:
                    addPrefixeInformation(prefix, attributes);
                    break;
                case FieldKey.SECTION_CATEGORY:
                    addPrefixedSection(prefix, attributes);
                    break;
            }
        }

        private void addPrefixedPropriete(String prefix, Attributes attributes) throws IOException {
            CorpusMetadata corpusMetadata = getCorpusMetatdata();
            for (CorpusField corpusField : corpusMetadata.getProprieteList()) {
                FieldKey fieldKey = corpusField.getFieldKey();
                FieldNameInfo fieldNameInfo = FieldNameInfo.testPrefix(fieldKey, prefix);
                if (fieldNameInfo != null) {
                    addPropriete(fieldKey, fieldNameInfo, getFieldOptionObject(corpusField, attributes));
                }
            }
        }

        private void addPrefixeInformation(String prefix, Attributes attributes) throws IOException {
            CorpusMetadata corpusMetadata = getCorpusMetatdata();
            for (CorpusField corpusField : corpusMetadata.getInformationList()) {
                FieldKey fieldKey = corpusField.getFieldKey();
                FieldNameInfo fieldNameInfo = FieldNameInfo.testPrefix(fieldKey, prefix);
                if (fieldNameInfo != null) {
                    addInformation(fieldKey, fieldNameInfo, getFieldOptionObject(corpusField, attributes));
                }
            }
        }

        private void addPrefixedSection(String prefix, Attributes attributes) throws IOException {
            CorpusMetadata corpusMetadata = getCorpusMetatdata();
            for (CorpusField corpusField : corpusMetadata.getSectionList()) {
                FieldKey fieldKey = corpusField.getFieldKey();
                FieldNameInfo fieldNameInfo = FieldNameInfo.testPrefix(fieldKey, prefix);
                if (fieldNameInfo != null) {
                    addSection(fieldKey, fieldNameInfo);
                }
            }
        }

        private void addTitre() throws IOException {
            String titre = (String) fichePointeur.getValue(FieldKey.TITRE);
            if (titre == null) {
                return;
            }
            openTag("titre");
            addSimpleElement("item", titre);
            closeTag("titre");
        }

        private void addSoustitre() throws IOException {
            Para para = (Para) fichePointeur.getValue(FieldKey.SOUSTITRE);
            if (para == null) {
                return;
            }
            openTag("soustitre");
            ficheItemXMLPart.addPara(para);
            closeTag("soustitre");
        }

        private void addRedacteurs() throws IOException {
            FicheItems ficheItems = (FicheItems) fichePointeur.getValue(FieldKey.REDACTEURS);
            if (ficheItems == null) {
                return;
            }
            openTag("redacteurs");
            ficheItemXMLPart.addFicheItems(ficheItems);
            closeTag("redacteurs");
        }

        private void addPropriete(FieldKey fieldKey, FieldNameInfo fieldNameInfo, Object fieldOptionObject) throws IOException {
            FicheItem ficheItem = (FicheItem) fichePointeur.getValue(fieldKey);
            if (ficheItem == null) {
                return;
            }
            startOpenTag("propriete");
            addAttribute("name", fieldKey.getFieldName());
            addAttribute("type", fieldKey.getFieldName());
            if (fieldNameInfo != null) {
                addAttribute("part1", fieldNameInfo.getPart1());
                addAttribute("part2", fieldNameInfo.getPart2());
            }
            endOpenTag();
            ficheItemXMLPart.addFicheItem(ficheItem, fieldOptionObject);
            closeTag("propriete");
        }

        private void addInformation(FieldKey fieldKey, FieldNameInfo fieldNameInfo, Object fieldOptionObject) throws IOException {
            FicheItems ficheItems = (FicheItems) fichePointeur.getValue(fieldKey);
            if (ficheItems == null) {
                return;
            }
            startOpenTag("information");
            addAttribute("name", fieldKey.getFieldName());
            addAttribute("type", fieldKey.getFieldName());
            if (fieldNameInfo != null) {
                addAttribute("part1", fieldNameInfo.getPart1());
                addAttribute("part2", fieldNameInfo.getPart2());
            }
            endOpenTag();
            ficheItemXMLPart.addFicheItems(ficheItems, fieldOptionObject);
            closeTag("information");
        }

        private void addSection(FieldKey fieldKey, FieldNameInfo fieldNameInfo) throws IOException {
            FicheBlocks blockList = (FicheBlocks) fichePointeur.getValue(fieldKey);
            if (blockList == null) {
                return;
            }
            startOpenTag("section");
            addAttribute("name", fieldKey.getFieldName());
            addAttribute("type", fieldKey.getFieldName());
            if (fieldNameInfo != null) {
                addAttribute("part1", fieldNameInfo.getPart1());
                addAttribute("part2", fieldNameInfo.getPart2());
            }
            endOpenTag();
            ficheBlockXMLPart.addFicheBlocks(blockList);
            closeTag("section");
        }

        private void addChrono() throws IOException {
            FicheMeta ficheMeta = (FicheMeta) fichePointeur.getCurrentSubsetItem();
            FuzzyDate creationDate = ficheMeta.getCreationDate();
            if (creationDate != null) {
                openTag("chrono");
                ficheItemXMLPart.addDatation(new Datation(creationDate), null);
                FuzzyDate modificationDate = ficheMeta.getModificationDate();
                if (!modificationDate.equals(creationDate)) {
                    ficheItemXMLPart.addDatation(new Datation(modificationDate), null);
                }
                closeTag("chrono");
            }
        }

        private void addLang() throws IOException {
            Lang lang = ((FicheMeta) fichePointeur.getCurrentSubsetItem()).getLang();
            if (lang != null) {
                openTag("lang");
                ficheItemXMLPart.addLangue(new Langue(lang));
                closeTag("lang");
            }
        }

        private void addFicheRattachement(FicheRattachementDef ficheRattachementDef) throws IOException {
            Corpus currentCorpus = fichePointeur.getCorpus();
            List<Corpus> corpusList = toCorpusList(currentCorpus, ficheRattachementDef);
            if (corpusList.isEmpty()) {
                return;
            }
            int id = fichePointeur.getCurrentSubsetItem().getId();
            FicheXMLPart ficheXMLPart = new FicheXMLPart(getXMLWriter(), extractionContext);
            FicheFilter ficheFilter = ficheRattachementDef.getFicheFilter();
            Predicate<FicheMeta> fichePredicate = extractionContext.getFichePredicate();
            if (fichePredicate == null) {
                fichePredicate = EligibilityUtils.ALL_FICHE_PREDICATE;
            }
            for (Corpus otherCorpus : corpusList) {
                if (!extractionContext.getSubsetAccessPredicate().test(otherCorpus)) {
                    continue;
                }
                FicheMeta ficheMeta = otherCorpus.getFicheMetaById(id);
                if ((ficheMeta != null) && (fichePredicate.test(ficheMeta))) {
                    FicheExtractInfo ficheExtractInfo = ExtractionXMLUtils.toFicheExtractInfo(ficheMeta, ficheFilter);
                    ficheXMLPart.addFiche(ficheExtractInfo);
                }
            }
        }

        private void addMotcleRattachement(MotcleRattachementDef motcleRattachementDef) throws IOException {
            Corpus currentCorpus = fichePointeur.getCorpus();
            Subset masterSubset = currentCorpus.getMasterSubset();
            if ((masterSubset == null) || (!(masterSubset instanceof Thesaurus))) {
                return;
            }
            if (!extractionContext.getSubsetAccessPredicate().test(masterSubset)) {
                return;
            }
            Motcle masterMotcle = (Motcle) masterSubset.getSubsetItemById(fichePointeur.getCurrentSubsetItem().getId());
            MotcleXMLPart motcleXMLPart = new MotcleXMLPart(getXMLWriter(), extractionContext);
            motcleXMLPart.addMotcle(ExtractionXMLUtils.toMotcleExtractInfo(masterMotcle, motcleRattachementDef.getMotcleFilter()));
        }

        private void addExtraitCorpus(ExtraitCorpusDef extraitCorpusDef) throws IOException {
            CorpusExtractor corpusExtractor = extractionContext.getExtractorProvider().getCorpusExtractor(extraitCorpusDef);
            Lang defaultLang = LocalisationUtils.getDefaultLang(extractionContext.getLangContext());
            SelectionContext selectionContext = SelectionContextBuilder.init(extractionContext, defaultLang)
                    .setSubsetAccessPredicate(extractionContext.getSubsetAccessPredicate())
                    .setFichePredicate(extractionContext.getFichePredicate())
                    .setCurrentCorpus(fichePointeur.getCorpus())
                    .toSelectionContext();
            FicheSelector ficheSelector = FicheSelectorBuilder.init(selectionContext)
                    .addAll(extraitCorpusDef.getFicheQueryList())
                    .toFicheSelector();
            SubsetItemPointeur pointeur = PointeurUtils.checkMasterPointeur(fichePointeur, extraitCorpusDef.isMaster());
            for (Corpus other : ficheSelector.getCorpusList()) {
                Croisements croisements = pointeur.getCroisements(other);
                for (Croisements.Entry entry : croisements.getEntryList()) {
                    FicheMeta ficheMeta2 = (FicheMeta) entry.getSubsetItem();
                    Croisement filteredCroisement = ficheSelector.isSelected(ficheMeta2, entry.getCroisement());
                    if (filteredCroisement != null) {
                        corpusExtractor.add(ficheMeta2, filteredCroisement);
                    }
                }
            }
            if (!corpusExtractor.containsFiche()) {
                return;
            }
            if (extraitCorpusXMLPart == null) {
                extraitCorpusXMLPart = new ExtraitCorpusXMLPart(getXMLWriter(), extractionContext);
            }
            for (ExtraitCorpusResult extraitCorpusResult : corpusExtractor.getExtraitCorpusResultList()) {
                extraitCorpusXMLPart.addExtraitcorpus(extraitCorpusResult);
            }
        }

        private void addExtraitThesaurus(ExtraitThesaurusDef extraitThesaurusDef, SelectionContext selectionContext) throws IOException {
            ThesaurusExtractor thesaurusExtractor = extractionContext.getExtractorProvider().getThesaurusExtractor(extraitThesaurusDef);
            MotcleSelector motcleSelector = MotcleSelectorBuilder.init(selectionContext).addAll(extraitThesaurusDef.getMotcleQueryList()).toMotcleSelector();
            SubsetItemPointeur pointeur = PointeurUtils.checkMasterPointeur(fichePointeur, extraitThesaurusDef.isMaster());
            for (Thesaurus thesaurus : motcleSelector.getThesaurusList()) {
                Croisements croisements = pointeur.getCroisements(thesaurus);
                for (Croisements.Entry entry : croisements.getEntryList()) {
                    Motcle motcle = (Motcle) entry.getSubsetItem();
                    Croisement filteredCroisement = motcleSelector.isSelected(motcle, entry.getCroisement());
                    if (filteredCroisement != null) {
                        thesaurusExtractor.add(motcle, filteredCroisement);
                    }
                }
            }
            if (!thesaurusExtractor.containsMotcle()) {
                return;
            }
            if (extraitThesaurusXMLPart == null) {
                extraitThesaurusXMLPart = new ExtraitThesaurusXMLPart(getXMLWriter(), extractionContext);
            }
            extraitThesaurusXMLPart.addExtraitThesaurus(thesaurusExtractor.getExtraitThesaurusResult());
        }

        private void addExtraitAlbum(ExtraitAlbumDef extraitAlbumDef, SelectionContext selectionContext) throws IOException {
            AlbumExtractor albumExtractor = extractionContext.getExtractorProvider().getAlbumExtractor(extraitAlbumDef);
            IllustrationSelector illustrationSelector = IllustrationSelectorBuilder.init(selectionContext).addAll(extraitAlbumDef.getIllustrationQueryList()).toIllustrationSelector();
            for (Album album : illustrationSelector.getAlbumList()) {
                Croisements croisements = fichePointeur.getCroisements(album);
                for (Croisements.Entry entry : croisements.getEntryList()) {
                    Illustration illustration = (Illustration) entry.getSubsetItem();
                    Croisement filteredCroisement = illustrationSelector.isSelected(illustration, entry.getCroisement());
                    if (filteredCroisement != null) {
                        albumExtractor.add(illustration, filteredCroisement);
                    }
                }
            }
            if (!albumExtractor.containsIllustration()) {
                return;
            }
            if (extraitAlbumXMLPart == null) {
                extraitAlbumXMLPart = new ExtraitAlbumXMLPart(getXMLWriter());
            }
            extraitAlbumXMLPart.addExtraitAlbum(albumExtractor.getExtraitAlbumResult());
        }

        private void addExtraitAddenda(ExtraitAddendaDef extraitAddendaDef, SelectionContext selectionContext) throws IOException {
            AddendaExtractor addendaExtractor = extractionContext.getExtractorProvider().getAddendaExtractor(extraitAddendaDef);
            DocumentSelector documentSelector = DocumentSelectorBuilder.init(selectionContext).addAll(extraitAddendaDef.getDocumentQueryList()).toDocumentSelector();
            for (Addenda addenda : documentSelector.getAddendaList()) {
                Croisements croisements = fichePointeur.getCroisements(addenda);
                for (Croisements.Entry entry : croisements.getEntryList()) {
                    Document document = (Document) entry.getSubsetItem();
                    Croisement filteredCroisement = documentSelector.isSelected(document, entry.getCroisement());
                    if (filteredCroisement != null) {
                        addendaExtractor.add(document, filteredCroisement);
                    }
                }
            }
            if (!addendaExtractor.containsDocument()) {
                return;
            }
            if (extraitAddendaXMLPart == null) {
                extraitAddendaXMLPart = new ExtraitAddendaXMLPart(getXMLWriter(), extractionContext);
            }
            extraitAddendaXMLPart.addExtraitAddenda(addendaExtractor.getExtraitAddendaResult());
        }

        private CorpusMetadata getCorpusMetatdata() {
            return fichePointeur.getCorpus().getCorpusMetadata();
        }

    }


    private static class LotXMLPart extends FillControllerXMLPart {

        private final String tagName;
        private final XMLWriter xmlWriter;
        private boolean openDone = false;

        private LotXMLPart(XMLWriter xmlWriter, LotFilterUnit lotFilterUnit) {
            super(xmlWriter);
            tagName = lotFilterUnit.getTagName();
            this.xmlWriter = xmlWriter;
        }

        @Override
        public void controlFill() throws IOException {
            if (tagName != null) {
                xmlWriter.openTag(tagName);
                openDone = true;
            }
        }

        private void end() throws IOException {
            if (tagName != null) {
                if (openDone) {
                    xmlWriter.closeTag(tagName);
                }
            }
        }

    }

    private static List<Corpus> toCorpusList(Corpus mainCorpus, FicheRattachementDef ficheRattachementDef) {
        List<SubsetKey> subsetKeyList = ficheRattachementDef.getRattachementSubsetKeyList();
        if (subsetKeyList.isEmpty()) {
            return FichothequeUtils.getRattachementCorpusList(mainCorpus);
        }
        Fichotheque fichotheque = mainCorpus.getFichotheque();
        Subset mainMasterSubset = mainCorpus.getMasterSubset();
        List<Corpus> list = new ArrayList<Corpus>();
        for (SubsetKey subsetKey : subsetKeyList) {
            Corpus corpus = (Corpus) fichotheque.getSubset(subsetKey);
            if ((corpus != null) && (!corpus.equals(mainCorpus))) {
                Subset masterSubset = corpus.getMasterSubset();
                if (mainMasterSubset == null) {
                    if ((masterSubset != null) && (masterSubset.equals(mainCorpus))) {
                        list.add(corpus);
                    }
                } else {
                    if ((corpus.equals(mainMasterSubset)) || ((masterSubset != null) && (masterSubset.equals(mainMasterSubset)))) {
                        list.add(corpus);
                    }
                }
            }
        }
        return list;
    }

    private static short typeToShort(String typeString) {
        switch (typeString) {
            case "day":
            case "jour":
                return FuzzyDate.DAY_TYPE;
            case "month":
            case "mois":
                return FuzzyDate.MONTH_TYPE;
            case "quarter":
            case "trimestre":
                return FuzzyDate.QUARTER_TYPE;
            case "halfyear":
            case "semestre":
                return FuzzyDate.HALFYEAR_TYPE;
            case "year":
            case "annee":
                return FuzzyDate.YEAR_TYPE;
            default:
                throw new IllegalArgumentException("unknown type = " + typeString);
        }
    }

}
